@@ -3,12 +3,9 @@
 #' Allocates an empty vector, matrix or array faster than the corresponding
 #' function in R.
 #'
-#'
 #' @param value A \code{\link[base]{numeric}} scalar that all elements will
 #' have as value.
 #'
-#' @param ... Not used.
-#'
 #' @param length,nrow,ncol,dim \code{\link[base]{numeric}}s specifying the
 #' dimension of the created \code{\link[base]{vector}},
 #' \code{\link[base]{matrix}} or \code{\link[base]{array}}.
@@ -34,7 +31,7 @@ allocMatrix <- function(nrow, ncol, value = 0.0, ...) {
 #' @rdname allocMatrix
 #' @export
 allocVector <- function(length, value = 0.0, ...) {
-  length <- as.integer(length)
+  length <- as.numeric(length)
   .Call(C_allocVector2, length, value)
 }
 
