@@ -2,19 +2,16 @@
 #'
 #' Computes the lagged and iterated differences.
 #'
+#' @inheritParams rowAlls
+#'
 #' @param x A \code{\link[base]{numeric}} \code{\link[base]{vector}} of length
 #' N.
 #'
-#' @param idxs A \code{\link[base]{vector}} indicating subset of elements to
-#' operate over. If \code{\link[base]{NULL}}, no subsetting is done.
-#'
 #' @param lag An \code{\link[base]{integer}} specifying the lag.
 #'
 #' @param differences An \code{\link[base]{integer}} specifying the order of
 #' difference.
 #'
-#' @param ... Not used.
-#'
 #' @return Returns a \code{\link[base]{numeric}} \code{\link[base]{vector}} of
 #' length N - \code{differences}.
 #'
@@ -33,12 +30,12 @@ diff2 <- function(x, idxs = NULL, lag = 1L, differences = 1L, ...) {
   # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
   # Argument 'lag':
   if (length(lag) != 1L) {
-    stop("Argument 'lag' is not a scalar: ", length(lag))
+    stop(sprintf("Argument '%s' is not a scalar: %.0f", "lag", length(lag)))
   }
 
   # Argument 'differences':
   if (length(differences) != 1L) {
-    stop("Argument 'differences' is not a scalar: ", length(differences))
+    stop(sprintf("Argument '%s' is not a scalar: %.0f", "differences", length(differences)))
   }
 
   lag <- as.integer(lag)
