@@ -3,21 +3,11 @@
 #' Extracts one cell per row (column) from a matrix.  The implementation is
 #' optimized for memory and speed.
 #'
-#' @param x An NxK \code{\link[base]{matrix}}.
+#' @inheritParams rowAlls
 #'
 #' @param idxs An index \code{\link[base]{vector}} of (maximum) length N (K)
 #' specifying the columns (rows) to be extracted.
 #'
-#' @param rows,cols A \code{\link[base]{vector}} indicating subset of rows
-#' (and/or columns) to operate over. If \code{\link[base]{NULL}}, no subsetting
-#' is done.
-#'
-#' @param dim. An \code{\link[base]{integer}} \code{\link[base]{vector}} of
-#' length two specifying the dimension of \code{x}, also when not a
-#' \code{\link[base]{matrix}}.
-#'
-#' @param ... Not used.
-#'
 #' @return Returns a \code{\link[base]{vector}} of length N (K).
 #'
 #' @example incl/rowCollapse.R
@@ -28,11 +18,9 @@
 #' cf. \code{\link[base]{[}}().
 #' @keywords utilities
 #' @export
-rowCollapse <- function(x, idxs, rows = NULL, dim. = dim(x), ...) {
+rowCollapse <- function(x, idxs, rows = NULL, dim. = dim(x), ..., useNames = NA) {
   # Argument 'x':
-  if (!is.matrix(x) && !is.vector(x)) {
-    .Defunct(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
-  }
+  if (!is.matrix(x) && !is.vector(x)) defunctShouldBeMatrixOrVector(x)
 
   # Argument 'idxs':
   idxs <- rep(idxs, length.out = dim.[1L])
@@ -54,17 +42,28 @@ rowCollapse <- function(x, idxs, rows = NULL, dim. = dim(x), ...) {
   # Calculate column-based indices
   idxs <- dim.[1L] * cols + seq_len(dim.[1L])
   cols <- NULL # Not needed anymore
+  
+  # Update names attribute?
+  res <- x[idxs]
+  if (!is.na(useNames)) {
+    if (useNames) {
+      names <- rownames(x)
+      if (!is.null(names)) {
+        names(res) <- names
+      }
+    } else {
+      names(res) <- NULL
+    }
+  }
 
-  x[idxs]
+  res
 }
 
 #' @rdname rowCollapse
 #' @export
-colCollapse <- function(x, idxs, cols = NULL, dim. = dim(x), ...) {
+colCollapse <- function(x, idxs, cols = NULL, dim. = dim(x), ..., useNames = NA) {
   # Argument 'x':
-  if (!is.matrix(x) && !is.vector(x)) {
-    .Defunct(msg = sprintf("Argument 'x' is of class %s, but should be a matrix or a vector. The use of a %s is not supported, the correctness of the result is not guaranteed. Please update your code accordingly.", sQuote(class(x)[1]), sQuote(class(x)[1])))  #nolint
-  }
+  if (!is.matrix(x) && !is.vector(x)) defunctShouldBeMatrixOrVector(x)
 
   # Argument 'idxs':
   idxs <- rep(idxs, length.out = dim.[2L])
@@ -86,6 +85,19 @@ colCollapse <- function(x, idxs, cols = NULL, dim. = dim(x), ...) {
   # Calculate column-based indices
   idxs <- dim.[1L] * 0:(dim.[2L] - 1L) + rows
   rows <- NULL # Not needed anymore
+  
+  # Update names attribute?
+  res <- x[idxs]
+  if (!is.na(useNames)) {
+    if (useNames) {
+      names <- colnames(x)
+      if (!is.null(names)) {
+        names(res) <- names
+      }
+    } else {
+      names(res) <- NULL
+    }
+  }
 
-  x[idxs]
+  res
 }
