@@ -2,26 +2,15 @@
 #'
 #' Gets the rank of the elements in each row (column) of a matrix.
 #'
-#' @param x A \code{\link[base]{numeric}} or \code{\link[base]{integer}} NxK
-#' \code{\link[base]{matrix}}.
-#'
-#' @param rows,cols A \code{\link[base]{vector}} indicating subset of rows
-#' (and/or columns) to operate over. If \code{\link[base]{NULL}}, no subsetting
-#' is done.
+#' @inheritParams rowAlls
 #'
 #' @param ties.method A \code{\link[base]{character}} string specifying how
 #' ties are treated.  For details, see below.
 #'
-#' @param dim. An \code{\link[base]{integer}} \code{\link[base]{vector}} of
-#' length two specifying the dimension of \code{x}, also when not a
-#' \code{\link[base]{matrix}}.
-#'
 #' @param preserveShape A \code{\link[base]{logical}} specifying whether the
 #' \code{\link[base]{matrix}} returned should preserve the input shape of
 #' \code{x}, or not.
 #'
-#' @param ... Not used.
-#'
 #' @return A \code{\link[base]{matrix}} of type \code{\link[base]{integer}} is
 #' returned, unless \code{ties.method = "average"} when it is of type
 #' \code{\link[base]{numeric}}.
@@ -99,19 +88,18 @@ rowRanks <- function(x, rows = NULL, cols = NULL,
                      # max is listed twice so that it remains the default for now
                      ties.method = c("max", "average", "first", "last", "random",
                                      "max", "min", "dense"),
-                     dim. = dim(x), ...) {
+                     dim. = dim(x), ..., useNames = NA) {
   # Argument 'ties.method':
   ties.method <- ties.method[1L]
 
   ties_method <- charmatch(ties.method, c("average", "first", "last", "random",
                                           "max", "min", "dense"), nomatch = 0L)
   if (ties_method == 0L) {
-    stop("Unknown value of argument 'ties.method': ", ties.method)
+    stop(sprintf("Unknown value of argument '%s': %s", "ties.method", ties.method))
   }
 
-  dim. <- as.integer(dim.)
   # byrow = TRUE
-  .Call(C_rowRanksWithTies, x, dim., rows, cols, ties_method, TRUE)
+  .Call(C_rowRanksWithTies, x, dim., rows, cols, ties_method, TRUE, useNames)
 }
 
 
@@ -121,7 +109,7 @@ colRanks <- function(x, rows = NULL, cols = NULL,
                      # max is listed twice so that it remains the default for now
                      ties.method = c("max", "average", "first", "last", "random",
                                      "max", "min", "dense"),
-                     dim. = dim(x), preserveShape = FALSE, ...) {
+                     dim. = dim(x), preserveShape = FALSE, ..., useNames = NA) {
   # Argument 'ties.method':
   ties.method <- ties.method[1L]
 
@@ -131,12 +119,12 @@ colRanks <- function(x, rows = NULL, cols = NULL,
   ties_method <- charmatch(ties.method, c("average", "first", "last", "random",
                                           "max", "min", "dense"), nomatch = 0L)
   if (ties_method == 0L) {
-    stop("Unknown value of argument 'ties.method': ", ties.method)
+    stop(sprintf("Unknown value of argument '%s': %s", "ties.method", ties.method))
   }
 
-  dim. <- as.integer(dim.)
   # byrow = FALSE
-  y <- .Call(C_rowRanksWithTies, x, dim., rows, cols, ties_method, FALSE)
+  y <- .Call(C_rowRanksWithTies, x, dim., rows, cols, ties_method, FALSE, useNames)
+
   if (!preserveShape) y <- t(y)
   y
 }
