@@ -26,14 +26,13 @@ SEXP mean2(SEXP x, SEXP idxs, SEXP naRm, SEXP refine) {
 
   /* Argument 'idxs': */
   R_xlen_t nidxs;
-  int idxsType;
-  void *cidxs = validateIndices(idxs, nx, 1, &nidxs, &idxsType);
+  R_xlen_t *cidxs = validateIndices(idxs, nx, 1, &nidxs);
 
   /* Double matrices are more common to use. */
   if (isReal(x)) {
-    avg = mean2_dbl[idxsType](REAL(x), nx, cidxs, nidxs, narm, refine2);
+    avg = mean2_dbl(REAL(x), nx, cidxs, nidxs, narm, refine2);
   } else if (isInteger(x) || isLogical(x)) {
-    avg = mean2_int[idxsType](INTEGER(x), nx, cidxs, nidxs, narm, refine2);
+    avg = mean2_int(INTEGER(x), nx, cidxs, nidxs, narm, refine2);
   }
 
   /* Return results */
