@@ -9,14 +9,18 @@
 #include <Rdefines.h>
 #include "000.types.h"
 #include "rowRanges_lowlevel.h"
+#include "naming.h"
 
-SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEXP hasNA) {
+SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEXP hasNA, SEXP useNames) {
   SEXP ans = NILSXP, ans2 = NILSXP;
   int *mins, *maxs;
   double *mins2, *maxs2;
   int *is_counted, all_counted = 0;
-  int what2, narm, hasna;
+  int what2, narm, hasna, usenames;
   R_xlen_t nrow, ncol, ii;
+  
+  /* Coercion moved down to C */
+  PROTECT(dim = coerceVector(dim, INTSXP));
 
   /* Argument 'x' and 'dim': */
   assertArgMatrix(x, dim, (R_TYPE_INT | R_TYPE_REAL), "x");
@@ -40,9 +44,11 @@ SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEX
 
   /* Argument 'rows' and 'cols': */
   R_xlen_t nrows, ncols;
-  int rowsType, colsType;
-  void *crows = validateIndices(rows, nrow, 0, &nrows, &rowsType);
-  void *ccols = validateIndices(cols, ncol, 0, &ncols, &colsType);
+  R_xlen_t *crows = validateIndices(rows, nrow, 0, &nrows);
+  R_xlen_t *ccols = validateIndices(cols, ncol, 0, &ncols);
+  
+  /* Argument 'useNames': */ 
+  usenames = asLogical(useNames);
 
   is_counted = (int *) R_alloc(nrows, sizeof(int));
 
@@ -52,7 +58,23 @@ SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEX
     } else {
       PROTECT(ans = allocVector(REALSXP, nrows));
     }
-    rowRanges_dbl[rowsType][colsType](REAL(x), nrow, ncol, crows, nrows, ccols, ncols, what2, narm, hasna, REAL(ans), is_counted);
+    rowRanges_dbl(REAL(x), nrow, ncol, crows, nrows, ccols, ncols, what2, narm, hasna, REAL(ans), is_counted);
+    if (usenames != NA_LOGICAL && usenames){
+      SEXP dimnames = getAttrib(x, R_DimNamesSymbol);
+      if (dimnames != R_NilValue) {
+        if (what2 == 2) {
+          if (nrows != 0) {
+            setDimnames(ans, dimnames, nrows, crows, 0, ccols, FALSE);
+          }
+          /* (else) Zero-length rownames attribute? Keep behavior same as base R function */
+        } else{
+          SEXP namesVec = VECTOR_ELT(dimnames, 0);
+          if (namesVec != R_NilValue) {
+            setNames(ans, namesVec, nrows, crows);
+          }        
+        }
+      }
+    }
     UNPROTECT(1);
   } else if (isInteger(x)) {
     if (what2 == 2) {
@@ -60,7 +82,7 @@ SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEX
     } else {
       PROTECT(ans = allocVector(INTSXP, nrows));
     }
-    rowRanges_int[rowsType][colsType](INTEGER(x), nrow, ncol, crows, nrows, ccols, ncols, what2, narm, hasna, INTEGER(ans), is_counted);
+    rowRanges_int(INTEGER(x), nrow, ncol, crows, nrows, ccols, ncols, what2, narm, hasna, INTEGER(ans), is_counted);
 
     /* Any entries with zero non-missing values? */
     all_counted = 1;
@@ -118,9 +140,27 @@ SEXP rowRanges(SEXP x, SEXP dim, SEXP rows, SEXP cols, SEXP what, SEXP naRm, SEX
 
       ans = ans2;
     }
+    if (usenames != NA_LOGICAL && usenames){
+      SEXP dimnames = getAttrib(x, R_DimNamesSymbol);
+      if (dimnames != R_NilValue) {
+        if (what2 == 2) {
+          if (nrows != 0) {
+            setDimnames(ans, dimnames, nrows, crows, 0, ccols, FALSE);
+          }
+          /* (else) Zero-length rownames attribute? Keep behavior same as base R function */
+        } else{
+          SEXP namesVec = VECTOR_ELT(dimnames, 0);
+          if (namesVec != R_NilValue) {
+            setNames(ans, namesVec, nrows, crows);
+          }        
+        }
+      }
+    }
 
     UNPROTECT(1); /* ans */
   }
+  
+  UNPROTECT(1); /* PROTECT(dim = ...) */
 
   return(ans);
 } // rowRanges()
